<?php

namespace Drupal\cdek_api;

use Psr\Http\Message\ResponseInterface;
use Drupal\Core\Serialization\Yaml;
use GuzzleHttp\Client;

/**
 * CDEK API request manager.
 */
class RequestManager {

  /**
   * CDEK API settings.
   *
   * @var array
   */
  protected $settings = [];

  /**
   * List of available endpoints.
   *
   * @var array
   */
  protected $endpoints = [];

  /**
   * The default http client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * RequestManager constructor.
   *
   * @param \GuzzleHttp\Client $http_client
   *   A guzzle http client instance.
   */
  public function __construct(Client $http_client) {
    $this->httpClient = $http_client;
    // Get module settings and endpoints.
    $this->settings = Cdek::getSettings();
    $this->endpoints = $this->getEndpoints();
  }

  /**
   * Send an "order_pickup_points" API request.
   *
   * @param array $params
   *   See \Drupal\cdek_api\Cdek::getOrderPickupPoints() for details.
   *
   * @return string|null
   *   Response body or NULL on failure.
   *
   * @see \Drupal\cdek_api\Cdek::getOrderPickupPoints()
   */
  public function sendOrderPickupPointsRequest(array $params = []) {
    $uri = $this->getEndpointUri('order_pickup_points');
    $options['query'] = $params;
    return $this->sendRequest('GET', $uri, $options);
  }

  /**
   * Send an "create_online_store_order" API request.
   *
   * @param string $data
   *   A string with XML data.
   *
   * @return string|null
   *   Response body or NULL on failure.
   */
  public function sendCreateOnlineStoreOrderRequest($data) {
    $uri = $this->getEndpointUri('create_online_store_order');
    $options['form_params'] = ['xml_request' => $data];
    return $this->sendRequest('POST', $uri, $options);
  }

  /**
   * Send an "create_delivery_order" API request.
   *
   * @param string $data
   *   A string with XML data.
   *
   * @return string|null
   *   Response body or NULL on failure.
   */
  public function sendCreateDeliveryOrderRequest($data) {
    $uri = $this->getEndpointUri('create_delivery_order');
    $options['form_params'] = ['xml_request' => $data];
    return $this->sendRequest('POST', $uri, $options);
  }

  /**
   * Send an "calculate_delivery_rate" API request.
   *
   * @param string $data
   *   A string with JSON data.
   *
   * @return string|null
   *   Response body or NULL on failure.
   *
   * @see \Drupal\cdek_api\Cdek::calculateDeliveryRate()
   */
  public function sendCalculateDeliveryRateRequest($data) {
    $uri = $this->getEndpointUri('calculate_delivery_rate');
    $options['headers'] = ['Content-Type' => 'application/json'];
    $options['body'] = $data;
    return $this->sendRequest('POST', $uri, $options);
  }

  /**
   * Send an "calculate_delivery_rate" API request.
   *
   * @param array $params
   *   See \Drupal\cdek_api\Cdek::calculateDeliveryRate() for details.
   *
   * @return string|null
   *   Response body or NULL on failure.
   *
   * @deprecated
   *   Use \Drupal\cdek_api\RequestManager::sendCalculateDeliveryRateRequest()
   *   instead.
   */
  public function sendDeliveryRateCalculationRequest(array $params = []) {
    return $this->sendCalculateDeliveryRateRequest(json_encode($params));
  }

  /**
   * Returns the endpoint URI.
   *
   * @param string $name
   *   The name of the endpoint.
   *
   * @return string
   *   The endpoint URI.
   */
  public function getEndpointUri($name) {
    $endpoint = $this->getEndpoint($name);
    return $this->getBasePath($endpoint['base_path']) . '/' . $endpoint['address'];
  }

  /**
   * Returns a list of available endpoints.
   *
   * @return array
   *   Array with endpoints.
   */
  protected function getEndpoints() {
    $filename = drupal_get_path('module', 'cdek_api') . '/cdek_api.endpoints.yml';
    return Yaml::decode(file_get_contents($filename));
  }

  /**
   * Returns a single endpoint.
   *
   * @param string $name
   *   The name of the endpoint.
   *
   * @return array
   *   The endpoint array.
   *
   * @throws \InvalidArgumentException
   *   When the endpoint does not exist.
   */
  protected function getEndpoint($name) {
    if (!isset($this->endpoints[$name])) {
      $message = sprintf('The "%s" endpoint does not exist.', $name);
      throw new \InvalidArgumentException($message);
    }
    return $this->endpoints[$name];
  }

  /**
   * Returns the base path for API request.
   *
   * @param string $type
   *   The type of base path.
   *
   * @return string
   *   The base path for API request.
   *
   * @throws \InvalidArgumentException
   *   When the base path does not exist.
   */
  protected function getBasePath($type) {
    if ($type != 'base_path' && $type != 'integration_base_path') {
      $message = sprintf('The "%s" base path does not exist.', $type);
      throw new \InvalidArgumentException($message);
    }
    return $this->settings[$type];
  }

  /**
   * Send an HTTP request.
   *
   * @param string $method
   *   HTTP method.
   * @param string $uri
   *   URI string.
   * @param array $options
   *   Request options to apply.
   *
   * @return string|null
   *   Response body or NULL on failure.
   */
  protected function sendRequest($method, $uri, array $options = []) {
    $options['timeout'] = (float) $this->settings['request_timeout'];
    $response = $this->httpClient->request($method, $uri, $options);
    return $this->isSuccessfulRequest($response) ? (string) $response->getBody() : NULL;
  }

  /**
   * Check the success of the HTTP request.
   *
   * @param \Psr\Http\Message\ResponseInterface $response
   *   Response of the HTTP request.
   *
   * @return bool
   *   The request is successful (TRUE) or not (FALSE).
   */
  protected function isSuccessfulRequest(ResponseInterface $response) {
    return $response->getStatusCode() >= 200 && $response->getStatusCode() < 300;
  }

}
